/***************************************************************************
 *
 * common.c : Common functions shared between modules
 *
 * Copyright (C) 2005 Richard Hughes, <richard@hughsie.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 **************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>
#include <gdk/gdkx.h>
#include <gnome.h>
#include <gconf/gconf-client.h>

#include "common.h"

void
g_log_ignore (const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
{
	/*
	 * This is a dummy function that is called only when not set verbose
	 * (and we shouldn't spew all the debug stuff)
	 */
}

/** Converts an action string representation to it's ENUM
 *
 *  @param  gconfstring		The string name
 *  @return			The action ENUM
 */
int
convert_string_to_policy (const gchar *gconfstring)
{
	g_assert (gconfstring);
	if (strcmp (gconfstring, "nothing") == 0)
		return ACTION_NOTHING;
	if (strcmp (gconfstring, "suspend") == 0)
		return ACTION_SUSPEND;
	if (strcmp (gconfstring, "shutdown") == 0)
		return ACTION_SHUTDOWN;
	if (strcmp (gconfstring, "hibernate") == 0)
		return ACTION_HIBERNATE;
	if (strcmp (gconfstring, "warning") == 0)
		return ACTION_WARNING;

	g_warning ("gconfstring '%s' not converted", gconfstring);
	return ACTION_UNKNOWN;
}

/** Converts an action ENUM to it's string representation
 *
 *  @param  powerDevice		The action ENUM
 *  @return			action string, e.g. "shutdown"
 */
GString *
convert_policy_to_string (gint value)
{
	GString *text;
	if (value == ACTION_NOTHING)
		text = g_string_new ("nothing");
	else if (value == ACTION_SUSPEND)
		text = g_string_new ("suspend");
	else if (value == ACTION_SHUTDOWN)
		text = g_string_new ("shutdown");
	else if (value == ACTION_HIBERNATE)
		text = g_string_new ("hibernate");
	else if (value == ACTION_WARNING)
		text = g_string_new ("warning");
	else {
		g_warning ("value '%i' not converted (unknown)", value);
		text = g_string_new ("unknown");
		}
	return text;
}

/** Converts an HAL string representation to it's ENUM
 *
 *  @param  gconfstring		The HAL battery type
 *  @return			The powerDevice ENUM
 */
gint convert_haltype_to_powerdevice (const gchar *type)
{
	g_assert (type);
	if (strcmp (type, "ac_adapter") == 0)
		return POWER_AC_ADAPTER;
	else if (strcmp (type, "ups") == 0)
		return POWER_UPS;
	else if (strcmp (type, "mouse") == 0)
		return POWER_MOUSE;
	else if (strcmp (type, "keyboard") == 0)
		return POWER_KEYBOARD;
	else if (strcmp (type, "pda") == 0)
		return POWER_PDA;
	else if (strcmp (type, "primary") == 0)
		return POWER_PRIMARY_BATTERY;
	return POWER_UNKNOWN;
}

/** Converts a powerDevice to it's hal representation (used for icons too)
 *
 *  @param  powerDevice		The powerDevice ENUM
 *  @return			HAL string, e.g. "primary"
 */
GString *
convert_powerdevice_to_haltype (gint powerDevice)
{
	GString *text;
	if (powerDevice == POWER_UPS)
		text = g_string_new ("ups");
	else if (powerDevice == POWER_MOUSE)
		text = g_string_new ("mouse");
	else if (powerDevice == POWER_KEYBOARD)
		text = g_string_new ("keyboard");
	else if (powerDevice == POWER_PRIMARY_BATTERY)
		text = g_string_new ("primary");
	else if (powerDevice == POWER_PDA)
		text = g_string_new ("pda");
	else
		text = g_string_new ("unknown");
	return text;
}

/** Converts a powerDevice to it's GNOME icon
 *
 *  @param  powerDevice		The powerDevice ENUM
 *  @return			GNOME Icon, e.g. "gnome-dev-pda"
 */
GString *
convert_powerdevice_to_gnomeicon (gint powerDevice)
{
	GString *text;
	if (powerDevice == POWER_UPS)
		text = NULL;
	else if (powerDevice == POWER_MOUSE)
		text = g_string_new ("gnome-dev-mouse-optical");
	else if (powerDevice == POWER_KEYBOARD)
		text = g_string_new ("gnome-dev-keyboard");
	else if (powerDevice == POWER_PRIMARY_BATTERY)
		text = g_string_new ("gnome-dev-battery");
	else if (powerDevice == POWER_PDA)
		text = g_string_new ("gnome-dev-pda");
	else if (powerDevice == POWER_UPS)
		text = g_string_new ("gnome-dev-ups");
	else
		text = NULL;
	return text;
}

/** Converts a powerDevice to it's human readable form
 *
 *  @param  powerDevice		The powerDevice ENUM
 *  @return			Human string, e.g. "Laptop battery"
 */
GString *
convert_powerdevice_to_string (gint powerDevice)
{
	GString *text;
	if (powerDevice == POWER_UPS)
		text = g_string_new ("UPS");
	else if (powerDevice == POWER_MOUSE)
		text = g_string_new ("Logitech mouse");
	else if (powerDevice == POWER_KEYBOARD)
		text = g_string_new ("Logitech keyboard");
	else if (powerDevice == POWER_PRIMARY_BATTERY)
		text = g_string_new ("Laptop battery");
	else if (powerDevice == POWER_PDA)
		text = g_string_new ("PDA");
	else
		text = g_string_new ("Unknown device");
	return text;
}

/** Updates .percentageCharge in a GenericObject
 *  This function is needed because of the different ways that batteries 
 *  can represent their charge.
 *
 *  @param  slotData		the GenericObject reference
 *  @param  datavalue 		Data value, either percentage or new mWh current
 */
void
update_percentage_charge (GenericObject *slotData)
{
	g_assert (slotData);
	g_assert (slotData->powerDevice != POWER_NONE);

	/* These devices cannot have charge, assume 0% */
	if (!slotData->present ||
	    slotData->powerDevice==POWER_AC_ADAPTER || 
	    slotData->powerDevice==POWER_UNKNOWN ) {
		slotData->percentageCharge = 0;
		return;
	}

	/* shouldn't happen */
	if (slotData->rawLastFull <= 0)
		slotData->rawLastFull = 100;

	/* 
	 * Work out the ACTUAL percentage charge of the battery 
	 * using cached values
	 */
	slotData->percentageCharge = ((double) slotData->rawCharge / 
					(double) slotData->rawLastFull) * 100;

	/* make sure results are sensible */
	if (slotData->percentageCharge < 0)
		slotData->percentageCharge = 0;
	if (slotData->percentageCharge > 100)
		slotData->percentageCharge = 100;
	return;
}

/** Gets the charge state string from a slot object
 *
 *  @param  slotData		the GenericObject reference
 *  @return			the charge string, e.g. "fully charged"
 */
GString *
get_chargestate_string (GenericObject *slotData)
{
	g_assert (slotData);
	GString *gs;
	if (!slotData->present)
		gs = g_string_new ("missing");
	else if (slotData->isCharging)
		gs = g_string_new ("charging");
	else if (slotData->isDischarging)
		gs = g_string_new ("discharging");
	else if (!slotData->isCharging && 
		 !slotData->isDischarging && 
		 slotData->percentageCharge > 98)
		gs = g_string_new ("fully charged");
	else if (!slotData->isCharging &&
		 !slotData->isDischarging)
		gs = g_string_new ("charged");
	else
		gs = g_string_new ("unknown");
	return gs;
}

/** Returns the time string, e.g. "2 hours 3 minutes"
 *
 *  @param  minutes		Minutes to convert to string
 *  @return			The timestring 
 */
GString *
get_timestring_from_minutes (gint minutes)
{
	/* g_debug ("get_timestring_from_minutes (%i)", minutes)); */
	GString* timestring;
	gint hours;				

	if (minutes == 0)
		return NULL;

	timestring = g_string_new ("");
	if (minutes == 1)
		g_string_printf (timestring, "%i minute", minutes);
	else if (minutes < 60)
		g_string_printf (timestring, "%i minutes", minutes);
	else {
		hours = minutes / 60;
		minutes = minutes - (hours * 60);
		if (minutes == 0) {
			if (hours == 1)
				g_string_printf (timestring, "%i hour", hours);
			else
				g_string_printf (timestring, "%i hours", hours);
		} else {
			if (hours == 1) {
				if (minutes == 1)
					g_string_printf (timestring, 
					"%i hour %i minute", hours, minutes);
				else
					g_string_printf (timestring, 
					"%i hour %i minutes", hours, minutes);
			} else
				g_string_printf (timestring, 
					"%i hours %i minutes", hours, minutes);
		}
	}
	return timestring;
}
