/***************************************************************************
 *
 * prefs.c : GNOME Power Preferences
 *
 * Copyright (C) 2005 Richard Hughes, <richard@hughsie.com>
 *
 * Taken in part from:
 * - GNOME Tutorial, example 1
 * http://www.gnome.org/~newren/tutorials/developing-with-gnome/html/apd.html
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 **************************************************************************/

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <gconf/gconf-client.h>

#include "common.h"
#include "prefs.h"

static GladeXML *all_pref_widgets;
static gboolean system_bus, isVerbose;
static HasData hasData;
gboolean displayIcon = TRUE;
gboolean displayIconLow = TRUE;

/** Sets/Hides GTK visibility
 *
 *  @param  widgetname		the libglade widget name
 *  @param  set			should widget be visible?
 */
static void
gtk_set_visibility (const char *widgetname, gboolean set)
{
	g_assert (widgetname);
	GtkWidget *widget;
	widget = glade_xml_get_widget (all_pref_widgets, widgetname);
	if (!widget) {
		g_warning ("widget '%s' failed to load, aborting", widgetname);
		return;
	}

	if (set)
		gtk_widget_show_all (widget);
	else
		gtk_widget_hide_all (widget);
}

/** Sets/Clears GTK Checkbox
 *
 *  @param  widgetname		the libglade widget name
 *  @param  set			should check be ticked?
 */
static void
gtk_set_check (const char *widgetname, gboolean set)
{
	g_assert (widgetname);
	GtkWidget *widget;
	widget = glade_xml_get_widget (all_pref_widgets, widgetname);
	if (!widget) {
		g_warning ("widget '%s' failed to load, aborting", widgetname);
		return;
	}
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), set);
}

/** Shows/hides/renames controls based on hasData, i.e. what hardware is in the system.
 *
 */
static void
recalc (void)
{
	GtkWidget *widget;
	/* checkboxes */
	gtk_set_check ("checkbutton_display_icon", displayIcon);
	gtk_set_check ("checkbutton_display_icon_low", displayIconLow);

	/* frame labels */
	if (hasData.hasBatteries) {
		widget = glade_xml_get_widget (all_pref_widgets, "label_frame_ac");
		gtk_label_set_markup (GTK_LABEL (widget), "<b>Running on AC adapter</b>");
		widget = glade_xml_get_widget (all_pref_widgets, "label_frame_batteries");
		gtk_label_set_markup (GTK_LABEL (widget), "<b>Running on batteries</b>");
	} else {
		widget = glade_xml_get_widget (all_pref_widgets, "label_frame_ac");
		gtk_label_set_markup (GTK_LABEL (widget), "<b>Configuration</b>");
	}
	
	/* top frame */
	gtk_set_visibility ("frame_batteries", hasData.hasBatteries);
	gtk_set_visibility ("combobox_battery_critical", hasData.hasBatteries);
	gtk_set_visibility ("label_battery_critical", hasData.hasBatteries);

	/* options */
	gtk_set_visibility ("combobox_button_lid", hasData.hasButtonLid);
	gtk_set_visibility ("label_button_lid", hasData.hasButtonLid);

	gtk_set_visibility ("combobox_ac_fail", hasData.hasAcAdapter);
	gtk_set_visibility ("label_ac_fail", hasData.hasAcAdapter);

	gtk_set_visibility ("combobox_ups_critical", hasData.hasUPS);
	gtk_set_visibility ("label_ups_critical", hasData.hasUPS);

	/* variables */
	gtk_set_visibility ("hscale_ac_brightness", hasData.hasLCD);
	gtk_set_visibility ("label_ac_brightness", hasData.hasLCD);
	gtk_set_visibility ("hscale_batteries_brightness", hasData.hasLCD);
	gtk_set_visibility ("label_batteries_brightness", hasData.hasLCD);

	gtk_set_visibility ("hscale_ac_display", hasData.hasDisplays);
	gtk_set_visibility ("label_ac_display", hasData.hasDisplays);
	gtk_set_visibility ("hscale_batteries_display", hasData.hasDisplays);
	gtk_set_visibility ("label_batteries_display", hasData.hasDisplays);

	gtk_set_visibility ("hscale_ac_hdd", hasData.hasHardDrive);
	gtk_set_visibility ("label_ac_hdd", hasData.hasHardDrive);
	gtk_set_visibility ("hscale_batteries_hdd", hasData.hasHardDrive);
	gtk_set_visibility ("label_batteries_hdd", hasData.hasHardDrive);
}

/** Perform the interactive action when a bool gconf key has been changed
 *
 *  @param  key			full gconf key path
 */
static void
gconf_key_action (const char *key)
{
	g_assert (key);
	gboolean value;
	GConfClient *client = gconf_client_get_default ();

	value = gconf_client_get_bool (client, key, NULL);

	if (strcmp (key, "/apps/gnome-power/general/hasUPS") == 0)
		hasData.hasUPS = value;
	else if (strcmp (key, "/apps/gnome-power/general/hasAcAdapter") == 0)
		hasData.hasAcAdapter = value;
	else if (strcmp (key, "/apps/gnome-power/general/hasBatteries") == 0)
		hasData.hasBatteries = value;
	else if (strcmp (key, "/apps/gnome-power/general/hasButtonPower") == 0)
		hasData.hasButtonPower = value;
	else if (strcmp (key, "/apps/gnome-power/general/hasButtonSleep") == 0)
		hasData.hasButtonSleep = value;
	else if (strcmp (key, "/apps/gnome-power/general/hasButtonLid") == 0)
		hasData.hasButtonLid = value;
	else if (strcmp (key, "/apps/gnome-power/general/hasDisplays") == 0)
		hasData.hasDisplays = value;
	else if (strcmp (key, "/apps/gnome-power/general/hasHardDrive") == 0)
		hasData.hasHardDrive = value;
	else if (strcmp (key, "/apps/gnome-power/general/hasLCD") == 0)
		hasData.hasLCD = value;
	else if (strcmp (key, "/apps/gnome-power/general/displayIcon") == 0)
		displayIcon = value;
	else if (strcmp (key, "/apps/gnome-power/general/displayIconLow") == 0)
		displayIconLow = value;
	else {
		g_warning ("Urecognised key [%s]", key);
		return;
	}
	recalc ();
}

/** Callback for gconf_key_changed
 *
 */
static void
callback_gconf_key_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
	g_assert (client);
	if (gconf_entry_get_value (entry) == NULL)
		return;

	if (gconf_entry_get_value (entry)->type == GCONF_VALUE_BOOL)
		gconf_key_action (entry->key);
}

/** gets gconf policy string from widget name, simple database
 *
 *  @param  widgetname		the libglade widget name
 *  @return 			the policy location
 */
static void
getPolicyFromWidgetname (const char *widgetname, GString **gconf, int *policytype)
{
	g_assert (widgetname);
	if (strcmp (widgetname, "combobox_sleep_type") == 0) {
		*gconf = g_string_new ("policy/SleepType");
		*policytype = POLICY_NONE;
	} else if (strcmp (widgetname, "combobox_button_power") == 0) {
		*gconf = g_string_new ("policy/ButtonPower");
		*policytype = POLICY_CHOICE;
	} else if (strcmp (widgetname, "combobox_button_suspend") == 0) {
		*gconf = g_string_new ("policy/ButtonSuspend");
		*policytype = POLICY_CHOICE;
	} else if (strcmp (widgetname, "combobox_button_lid") == 0) {
		*gconf = g_string_new ("policy/ButtonLid");
		*policytype = POLICY_CHOICE;
	} else if (strcmp (widgetname, "combobox_ac_fail") == 0) {
		*gconf = g_string_new ("policy/ACFail");
		*policytype = POLICY_CHOICE;
	} else if (strcmp (widgetname, "combobox_battery_critical") == 0) {
		*gconf = g_string_new ("policy/BatteryCritical");
		*policytype = POLICY_CHOICE;
	} else if (strcmp (widgetname, "combobox_ups_critical") == 0) {
		*gconf = g_string_new ("policy/UPSCritical");
		*policytype = POLICY_CHOICE;
	} else if (strcmp (widgetname, "hscale_ac_computer") == 0) {
		*gconf = g_string_new ("policy/AC/SleepComputer");
		*policytype = POLICY_TIME;
	} else if (strcmp (widgetname, "hscale_ac_hdd") == 0) {
		*gconf = g_string_new ("policy/AC/SleepHardDrive");
		*policytype = POLICY_TIME;
	} else if (strcmp (widgetname, "hscale_ac_display") == 0) {
		*gconf = g_string_new ("policy/AC/SleepDisplay");
		*policytype = POLICY_TIME;
	} else if (strcmp (widgetname, "hscale_ac_brightness") == 0) {
		*gconf = g_string_new ("policy/AC/Brightness");
		*policytype = POLICY_PERCENT;
	} else if (strcmp (widgetname, "hscale_batteries_computer") == 0) {
		*gconf = g_string_new ("policy/Batteries/SleepComputer");
		*policytype = POLICY_TIME;
	} else if (strcmp (widgetname, "hscale_batteries_hdd") == 0) {
		*gconf = g_string_new ("policy/Batteries/SleepHardDrive");
		*policytype = POLICY_TIME;
	} else if (strcmp (widgetname, "hscale_batteries_display") == 0) {
		*gconf = g_string_new ("policy/Batteries/SleepDisplay");
		*policytype = POLICY_TIME;
	} else if (strcmp (widgetname, "hscale_batteries_brightness") == 0) {
		*gconf = g_string_new ("policy/Batteries/Brightness");
		*policytype = POLICY_PERCENT;
	} else if (strcmp (widgetname, "checkbutton_display_icon") == 0) {
		*gconf = g_string_new ("general/displayIcon");
		*policytype = POLICY_BOOLEAN;
	} else if (strcmp (widgetname, "checkbutton_display_icon_low") == 0) {
		*gconf = g_string_new ("general/displayIconLow");
		*policytype = POLICY_BOOLEAN;
	} else if (strcmp (widgetname, "hscale_critical_low") == 0) {
		*gconf = g_string_new ("general/lowThreshold");
		*policytype = POLICY_PERCENT;
	} else {
		g_warning ("getPolicyFromWidgetname failed in it's lookup of '%s'!", widgetname);
		*gconf = NULL;
		*policytype = POLICY_NONE;
	}
}

/** Callback for combo_changed
 *
 */
static void
callback_combo_changed (GtkWidget *widget, gpointer user_data)
{
	g_assert (widget);
	gint value;
	const char *widgetname;
	GConfClient *client = gconf_client_get_default ();
	gchar *gconfpath;
	GString *policydata;
	GString *policyoption;
	gint policytype;

	value = gtk_combo_box_get_active(GTK_COMBO_BOX (widget));	
	widgetname = gtk_widget_get_name(widget);

	/* find policy name from gtk widgetname */
	getPolicyFromWidgetname (widgetname, &policydata, &policytype);
	if (!policydata) {
		g_warning ("Widget '%s' not found!", widgetname);
		return;
	}

	/* if property all okay, then prepend with gconfpath */
	gconfpath = g_strconcat ("/apps/gnome-power/", policydata->str, NULL);
	g_warning ("'%s' -> [%s] = (%i)", widgetname, gconfpath, value);
	policyoption = convert_policy_to_string (value);
	
	/* save to gconf */
	gconf_client_set_string (client, gconfpath, policyoption->str, NULL);
	g_string_free (policyoption, TRUE);
	g_string_free (policydata, TRUE);
	g_free (gconfpath);
	
	return;
}

/** Callback for hscale_changed
 *
 */
static void
callback_hscale_changed (GtkWidget *widget, gpointer user_data)
{
	g_assert (widget);
	gint value;
	const char *widgetname;
	GConfClient *client = gconf_client_get_default ();
	gchar *gconfpath;
	GString *policydata;
	gint policytype;

	value = (int) gtk_range_get_value (GTK_RANGE (widget));	
	widgetname = gtk_widget_get_name(widget);

	/* find policy name from gtk widgetname */
	getPolicyFromWidgetname (widgetname, &policydata, &policytype);
	if (!policydata) {
		g_warning ("Widget '%s' not found!", widgetname);
		return;
	}

	/* if property all okay, then prepend with gconfpath */
	gconfpath = g_strconcat ("/apps/gnome-power/", policydata->str, NULL);
	g_debug ("'%s' -> [%s] = (%i)", widgetname, gconfpath, value);

	/* save to gconf */
	gconf_client_set_int (client, gconfpath, value, NULL);
	g_free (gconfpath);
	if (policydata)
		g_string_free (policydata, TRUE);

	return;
}

/** Callback for button_help
 *
 */
static void
callback_help (GtkWidget *widget, gpointer user_data)
{
	/* for now, show website */
	gnome_url_show ("http://gnome-power.sourceforge.net/", NULL);
}

/** Callback for check_changed
 *
 */
static void
callback_check_changed (GtkWidget *widget, gpointer user_data)
{
	g_assert (widget);
	gboolean value;
	const char *widgetname;
	GConfClient *client = gconf_client_get_default ();
	gchar *gconfpath;
	GString *policydata;
	gint policytype;

	widgetname = gtk_widget_get_name(widget);
	value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));	

	/* find policy name from gtk widgetname */
	getPolicyFromWidgetname (widgetname, &policydata, &policytype);
	if (!policydata) {
		g_warning ("Widget '%s' not found!", widgetname);
		return;
	}

	/* if property all okay, then prepend with gconfpath */
	gconfpath = g_strconcat ("/apps/gnome-power/", policydata->str, NULL);
	g_debug ("'%s' -> [%s] = (%i)", widgetname, gconfpath, value);

	/* save to gconf */
	gconf_client_set_bool (client, gconfpath, value, NULL);
	g_free (gconfpath);
	if (policydata)
		g_string_free (policydata, TRUE);
}

/** Prints program usage.
 *
 */
static void
print_usage (void)
{
	g_print ("\nusage : gnome-power-preferences [--system-bus] [--verbose] [--help]\n");
	g_print (
		"\n"
		"        --system-bus            Use the gconf system bus\n"
		"        --verbose               Show extra debugging\n"
		"        --help                  Show this information and exit\n"
		"\n");
}

/** simple callback just appending a "%" to the reading
 *
 */
static gchar*
format_value_callback_percent (GtkScale *scale, gdouble value)
{
	return g_strdup_printf ("%i%%", (int) value);
}

/** simple callback that converts minutes into pretty text
 *
 */
static gchar*
format_value_callback_time (GtkScale *scale, gdouble value)
{
	char unitstring[32];
	GString *strvalue;
	if (value == 0)
		return g_strdup_printf ("Never");
		
	strvalue = get_timestring_from_minutes (value);
	strcpy (unitstring, strvalue->str);
	g_string_free (strvalue, TRUE);
	return g_strdup_printf ("%s", unitstring);
}



/** Sets the comboboxes up to the gconf value, and sets up callbacks.
 *
 *  @param  widgetname		the libglade widget name
 */
static void
combo_setup_action (const char *widgetname)
{
	g_assert (widgetname);
	GtkWidget *widget;
	GConfClient *client = gconf_client_get_default ();
	gint value, policytype;
	gchar *gconfpath;
	GString *policydata;
	gchar *policyoption;

	/* find policy name from gtk widgetname */
	getPolicyFromWidgetname (widgetname, &policydata, &policytype);
	if (!policydata) {
		g_warning ("Widget '%s' not found!", widgetname);
		return;
	}

	widget = glade_xml_get_widget (all_pref_widgets, widgetname);
	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), "Do nothing");
	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), "Send warning");
	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), "Suspend");
	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), "Hibernate");
	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), "Shutdown");

	gconfpath = g_strconcat ("/apps/gnome-power/", policydata->str, NULL);
	policyoption = gconf_client_get_string (client, gconfpath, NULL);
	g_free (gconfpath);
	if (policydata)
		g_string_free (policydata, TRUE);

	if (!policyoption) {
		g_warning ("gconf_client_get_string for widget '%s' failed (policy='%s')!!", widgetname, policydata->str);
		return;
	}

	value = convert_string_to_policy (policyoption);
	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), value);
	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (callback_combo_changed), NULL);
}

/** Sets the hscales up to the gconf value, and sets up callbacks.
 *
 *  @param  widgetname		the libglade widget name
 */
static void
hscale_setup_action (const char *widgetname)
{
	g_assert (widgetname);
	GtkWidget *widget;
	GConfClient *client = gconf_client_get_default ();
	gint value, policytype;
	gchar *gconfpath;
	GString *policydata;

	widget = glade_xml_get_widget (all_pref_widgets, widgetname);

	/* find policy name from gtk widgetname */
	getPolicyFromWidgetname (widgetname, &policydata, &policytype);

	if (!policydata) {
		g_warning ("Widget '%s' not found!", widgetname);
		return;
	}

	gconfpath = g_strconcat ("/apps/gnome-power/", policydata->str, NULL);
	value = gconf_client_get_int (client, gconfpath, NULL);
	g_debug ("'%s' -> [%s] = (%i)", widgetname, gconfpath, value);
	g_free (gconfpath);
	if (policydata)
		g_string_free (policydata, TRUE);

	/*if (strcmp (policydata, "policy/AC/Brightness") == 0 ||
	    strcmp (policydata, "policy/AC/Brightness") == 0)*/
	if (policytype == POLICY_PERCENT)
		g_signal_connect (G_OBJECT (widget), "format-value", G_CALLBACK (format_value_callback_percent), NULL);
	else
		g_signal_connect (G_OBJECT (widget), "format-value", G_CALLBACK (format_value_callback_time), NULL);
	gtk_range_set_value (GTK_RANGE (widget), (int) value);
	g_signal_connect (G_OBJECT (widget), "value-changed", G_CALLBACK (callback_hscale_changed), NULL);
}

/** Main program
 *
 */
int
main (int argc, char **argv)
{
	gtk_init (&argc, &argv);
	gconf_init (argc, argv, NULL);
	GtkWidget *widget = NULL;
	gint a, value;

	system_bus = FALSE;
	isVerbose = FALSE;
	for (a=1; a < argc; a++) {
		if (strcmp (argv[a], "--verbose") == 0)
			isVerbose = TRUE;
		else if (strcmp (argv[a], "--system-bus") == 0)
			system_bus = TRUE;
		else if (strcmp (argv[a], "--help") == 0) {
			print_usage ();
			return EXIT_SUCCESS;
		}
	}

	if (!isVerbose)
		g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, g_log_ignore, NULL);

	/* load the interface */
	all_pref_widgets = glade_xml_new (GPM_DATA "preferences.glade", NULL, NULL);
	if (!all_pref_widgets) {
		g_print ("glade file failed to load, aborting\n");
		return 0;
	}

	/* Get the GconfClient, tell it we want to monitor /apps/gnome-power */
	GConfClient *client = gconf_client_get_default ();
	gconf_client_add_dir (client, "/apps/gnome-power", GCONF_CLIENT_PRELOAD_NONE, NULL);
	gconf_client_notify_add (client, "/apps/gnome-power", callback_gconf_key_changed, widget, NULL, NULL);

	/* Get the main_window quit */
	widget = glade_xml_get_widget (all_pref_widgets, "window_preferences");
	if (!widget)
		g_error ("Main window failed to load, aborting");
	g_signal_connect (G_OBJECT (widget), "delete_event", G_CALLBACK (gtk_main_quit), NULL);

	/* Get the quit button */
	widget = glade_xml_get_widget (all_pref_widgets, "button_close");
	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (gtk_main_quit), NULL);

	/* Get the help button */
	widget = glade_xml_get_widget (all_pref_widgets, "button_help");
	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (callback_help), NULL);

	/* Get the display icon checkbox callbacks */
	widget = glade_xml_get_widget (all_pref_widgets, "checkbutton_display_icon");
	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (callback_check_changed), NULL);
	widget = glade_xml_get_widget (all_pref_widgets, "checkbutton_display_icon_low");
	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (callback_check_changed), NULL);

	/* set up comboboxes dynamically */
	combo_setup_action ("combobox_button_power");
	combo_setup_action ("combobox_button_suspend");
	combo_setup_action ("combobox_button_lid");
	combo_setup_action ("combobox_ac_fail");
	combo_setup_action ("combobox_battery_critical");
	combo_setup_action ("combobox_ups_critical");

	/* SleepType checkbox */
	widget = glade_xml_get_widget (all_pref_widgets, "combobox_sleep_type");
	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), "Quick sleep");
	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), "Hibernate");
	value = gconf_client_get_int (client, "/apps/gnome-power/general/SleepType", NULL);
	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), value);
	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (callback_combo_changed), NULL);

	/* sliders */
	hscale_setup_action ("hscale_ac_computer");
	hscale_setup_action ("hscale_ac_hdd");
	hscale_setup_action ("hscale_ac_display");
	hscale_setup_action ("hscale_ac_brightness");
	hscale_setup_action ("hscale_batteries_computer");
	hscale_setup_action ("hscale_batteries_hdd");
	hscale_setup_action ("hscale_batteries_display");
	hscale_setup_action ("hscale_batteries_brightness");
	hscale_setup_action ("hscale_critical_low");

	gconf_key_action ("/apps/gnome-power/general/displayIcon");
	gconf_key_action ("/apps/gnome-power/general/displayIconLow");
	gconf_key_action ("/apps/gnome-power/general/hasHardDrive");
	gconf_key_action ("/apps/gnome-power/general/hasBatteries");
	gconf_key_action ("/apps/gnome-power/general/hasAcAdapter");
	gconf_key_action ("/apps/gnome-power/general/hasUPS");
	gconf_key_action ("/apps/gnome-power/general/hasLCD");
	gconf_key_action ("/apps/gnome-power/general/hasDisplays");
	gconf_key_action ("/apps/gnome-power/general/hasUPS");
	gconf_key_action ("/apps/gnome-power/general/hasButtonLid");
	gconf_key_action ("/apps/gnome-power/general/hasButtonSleep");
	gconf_key_action ("/apps/gnome-power/general/hasButtonPower");

	/* start the event loop */
	gtk_main ();

	return 0;
}
